@@ -15,6 +15,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *  Copyright (C) 1999-2004 Olivier Fourdan (fourdan@xfce.org)
+ *  Copyright (C) 2011 Peter de Ridder (peter@xfce.org)
  *
  *  Portions based Thinice port by
  *                       Tim Gerla <timg@rrv.net>,
@@ -65,6 +66,7 @@ typedef enum
     CHECK_BASE,
     CHECK_TEXT,
     CHECK_CROSS,
+    CHECK_DASH,
     RADIO_LIGHT,
     RADIO_DARK,
     RADIO_BASE,
@@ -99,6 +101,11 @@ static const guchar check_cross_bits[] = {
     0xf8, 0x00, 0x78, 0x00, 0x38, 0x00, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00,
 };
+static const guchar check_dash_bits[] = {
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x03,
+    0xf8, 0x03, 0xf8, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00,
+};
 static const guchar radio_light_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x08, 0x00, 0x08,
     0x00, 0x08, 0x00, 0x08, 0x00, 0x08, 0x00, 0x04, 0x08, 0x02, 0xf0, 0x01,
@@ -136,6 +143,7 @@ parts[] =
     { check_base_bits,  NULL },
     { check_text_bits,  NULL },
     { check_cross_bits, NULL },
+    { check_dash_bits,  NULL },
     { radio_light_bits, NULL },
     { radio_dark_bits,  NULL },
     { radio_base_bits,  NULL },
@@ -505,6 +513,23 @@ static void draw_shadow(GtkStyle * style, GdkWindow * window, GtkStateType state
     g_return_if_fail(style != NULL);
     g_return_if_fail(window != NULL);
 
+    /* Spin buttons are a special case */
+    if (widget && GTK_IS_SPIN_BUTTON (widget))
+    {
+        /* GtkEntry doesn't provide the text area as frame size */
+        if (DETAIL("entry"))
+        {
+            gint line_width = 0;
+            gboolean interior_focus;
+            gdk_drawable_get_size (GTK_ENTRY(widget)->text_area, &width, NULL);
+            if (!gtk_widget_has_focus(widget))
+                gtk_widget_style_get(widget, "focus-line-width", &line_width, "interior-focus", &interior_focus, NULL);
+            if (interior_focus)
+                line_width = 0;
+            width += (style->xthickness + line_width) * 2;
+        }
+    }
+
     if (shadow_type == GTK_SHADOW_NONE)
         return;
 
@@ -665,7 +690,7 @@ static void draw_shadow(GtkStyle * style, GdkWindow * window, GtkStateType state
         case GTK_SHADOW_OUT:
             if (XFCE_RC_STYLE(style->rc_style)->smooth_edge)
             {
-                if ((DETAIL("spinbutton_up") || DETAIL("spinbutton_up") || DETAIL("spinbutton_down")) && (state_type != GTK_STATE_PRELIGHT))
+                if ((DETAIL("spinbutton_up") || DETAIL("spinbutton_down")) && (state_type != GTK_STATE_PRELIGHT))
                 {
                     /* Do nothing */
                 }
@@ -807,10 +832,6 @@ static void draw_shadow(GtkStyle * style, GdkWindow * window, GtkStateType state
             }
             else
             {
-                if ((DETAIL("spinbutton_up") || DETAIL("spinbutton_down")) && (state_type != GTK_STATE_PRELIGHT))
-                {
-                    /* Do nothing */
-                }
                 if ((DETAIL("spinbutton_up") || DETAIL("spinbutton_down")) && (state_type != GTK_STATE_PRELIGHT))
                 {
                     /* Do nothing */
@@ -968,6 +989,10 @@ static void draw_check(GtkStyle * style, GdkWindow * window, GtkStateType state,
         {
             draw_part(window, style->fg_gc[state], area, x, y, CHECK_CROSS);
         }
+        else if (shadow == GTK_SHADOW_ETCHED_IN)
+        {
+            draw_part(window, style->fg_gc[state], area, x, y, CHECK_DASH);
+        }
     }
     else
     {
@@ -979,6 +1004,10 @@ static void draw_check(GtkStyle * style, GdkWindow * window, GtkStateType state,
         {
             draw_part(window, style->text_gc[state], area, x, y, CHECK_CROSS);
         }
+        else if (shadow == GTK_SHADOW_ETCHED_IN)
+        {
+            draw_part(window, style->fg_gc[state], area, x, y, CHECK_DASH);
+        }
     }
 }
 
@@ -1310,6 +1339,27 @@ static void draw_extension(GtkStyle * style, GdkWindow * window, GtkStateType st
     }
 }
 
+static void draw_focus(GtkStyle * style, GdkWindow * window, GtkStateType state_type, GdkRectangle * area, GtkWidget * widget, const gchar * detail, gint x, gint y, gint width, gint height)
+{
+    /* Spin buttons are a special case */
+    if (widget && GTK_IS_SPIN_BUTTON (widget))
+    {
+        /* GtkEntry doesn't provide the text area as focus size */
+        if (DETAIL("entry"))
+        {
+            gint line_width = 0;
+            gboolean interior_focus;
+            gdk_drawable_get_size (GTK_ENTRY(widget)->text_area, &width, NULL);
+            gtk_widget_style_get (widget, "focus-line-width", &line_width, "interior-focus", &interior_focus, NULL);
+            if (interior_focus)
+                line_width = 0;
+            width += (style->xthickness + line_width) * 2;
+        }
+    }
+
+    parent_class->draw_focus(style, window, state_type, area, widget, detail, x, y, width, height);
+}
+
 static void draw_slider(GtkStyle * style, GdkWindow * window, GtkStateType state_type, GtkShadowType shadow_type, GdkRectangle * area, GtkWidget * widget, const gchar * detail, gint x, gint y, gint width, gint height, GtkOrientation orientation)
 {
     g_return_if_fail(style != NULL);
@@ -1389,6 +1439,7 @@ static void xfce_style_class_init(XfceStyleClass * klass)
     style_class->draw_shadow_gap = draw_shadow_gap;
     style_class->draw_box_gap = draw_box_gap;
     style_class->draw_extension = draw_extension;
+    style_class->draw_focus = draw_focus;
     style_class->draw_slider = draw_slider;
     style_class->draw_handle = draw_handle;
 }
